@@ -89,14 +89,17 @@ class HydratCmdln(cmdln.Cmdln):
 
     for module in CONFIG_IMPORTS:
       logger.debug("importing {0}".format(module))
-      importlib.import_module('.' + module, 'hydrat')
+      try:
+        importlib.import_module('.' + module, 'hydrat')
+      except ImportError:
+        logger.warning("failed to import {0} when configuring hydrat".format(module))
 
     configurables = set(configuration.Configurable.__subclasses__())
     logger.info("found {0} Configurable subclasses".format(len(configurables)))
     
     for item in configurables:
       logger.debug("configuring {0}".format(item.__name__))
-      config = configuration.update_configuration(config, item.requires, rescan=opts.rescan, scan=opts.scan)
+      config = configuration.update_configuration(config, item.requires, rescan=opts.rescan, scan=opts.scan, blank_keys=True)
 
     configuration.write_configuration(config, path)
     logger.info("Wrote configuration file to '%s'", path)
