@@ -5,7 +5,7 @@ from numpy import seterr
 from numpy import concatenate
 from numpy import logical_xor
 from numpy.linalg import norm
-from hydrat.common import progress, timed_report
+from hydrat.common import progress, timed_report, rankdata
 from hydrat.common.pb import ProgressIter
 
 def dot(q,r):
@@ -27,6 +27,7 @@ class NullVector(MetricError):
     self.vector_num = vector_num
     self.index = index
 
+# TODO: Rename to DistanceMetric for consistency
 class distance_metric(object):
   """Abstract parent for distance metric classes"""
   __name__ = "distance_metric"
@@ -90,7 +91,11 @@ class dm_cosine(distance_metric):
     return 1 - results
 
 class dm_skew(distance_metric):
-  # TODO: Implement dimensionality reduction as per cosine.
+  """
+  Skew Divergence
+  See Lee L. Measures of distributional similarity. 
+  In: Proceedings of the 37th Annual Meeting of the ACL. College Park, USA; 1999:25-32.
+  """
   __name__ = "skew"
 
   def __init__(self, alpha = 0.99):
@@ -103,121 +108,91 @@ class dm_skew(distance_metric):
     return p
 
   def vector_distances(self, v1, v2, thresh=10):
-    # TODO: There is a problem in this implementation.
-    #       Vector distance calculation outcomes are different if we do
-    #       v1xv2 versus one member of v2 at a time against v1.
-    #       The issue is probably in the feature collapse step, we are possibly collapsing too many
-    #       features, or normalizing wrongly as was the case with cosine.
-    #       This problem becomes evident when either v1 or v2 is very short, and so we keep very few features.
-    #       As a temporary workaround, we skip the feature collapse if v1 or v2 are below a threshold
     self.logger.debug('Creating dense representation')
     orig_feats = v1.shape[1]
+    k = numpy.log(1 / (1 - self.alpha))
+
+    # Calculate instance sums
+    sum1 = numpy.array(v1.sum(1))
+    sum2 = numpy.array(v2.sum(1))
 
     # Determine shared features
-    if v1.shape[0] > thresh and v2.shape[0] > thresh:
-      f1 = numpy.asarray( v1.sum(0) ) [0] > 0
-      f2 = numpy.asarray( v2.sum(0) ) [0] > 0
-      i  = numpy.flatnonzero(numpy.logical_and(f1, f2))
+    f1 = numpy.asarray( v1.sum(0) ) [0] > 0
+    f2 = numpy.asarray( v2.sum(0) ) [0] > 0
+    nb = numpy.flatnonzero(numpy.logical_and(f1, f2)) # both nonzero
+    n1 = numpy.flatnonzero(numpy.logical_and(f1, numpy.logical_not(f2))) # only f1 nonzero
       
-      if len(i) == 0:
-        # No overlapping features.
-        return numpy.empty((v1.shape[0],v2.shape[0]), dtype='float')
+    if len(nb) == 0:
+      # No overlapping features.
+      return numpy.empty((v1.shape[0],v2.shape[0]), dtype='float')
 
-      # Select only shared features from both matrices
-      v1 = v1.transpose()[i].transpose().toarray()
-      v2 = v2.transpose()[i].transpose().toarray()
-      self.logger.debug('Reduced matrices from %d to %d features', orig_feats, v1.shape[1] )
+    # Calculate term for where v2 is always zero but v1 is not
+    #   This is a 1-dimensional vector, one scalar value per v1 instance
+    #   In the case where there are no such terms, we set penalty to 0 and let array
+    #   broadcasting take care of it.
+    if len(n1) > 0:
+      penalty = numpy.array(v1[:,n1].sum(1))
     else:
-      v1 = v1.toarray()
-      v2 = v2.toarray()
-      self.logger.debug('Maintaining full set of %d features', orig_feats )
+      penalty = 0
 
-    self.logger.debug('Calculating distributions')
-    s1 = [ float(v.sum()) for v in v1 ]
-    s2 = [ float(v.sum()) for v in v2 ]
-    # Replace an empty distribution with a uniform distribution.
-    # Empty instances are a problem all the way back at the dataset level.
-    n1 = [ self.alpha * ( (v/s) if s > 0 else numpy.ones_like(v) ) for (v,s) in izip(v1, s1) ]
-    n2 = [ ( (v/s) if s > 0 else numpy.ones_like(v) ) for (v,s) in izip(v2,s2) ]
-    n3 = [ (1-self.alpha) * q for q in n2 ]
+    # Select only shared features from both matrices
+    v1 = v1.transpose()[nb].transpose().toarray()
+    v2 = v2.transpose()[nb].transpose().toarray()
+    self.logger.debug('Reduced matrices from %d to %d features', orig_feats, v1.shape[1] )
 
-    self.logger.debug('Computing Distances')
-    results = numpy.empty((v1.shape[0],v2.shape[0]), dtype='float')
+    # Replace empty distributions with uniform distributions
+    for i in numpy.flatnonzero(sum1 == 0):
+      v1[i] = numpy.ones_like(v1[i])
+      sum1[i] = v1[i].sum()
+    for i in numpy.flatnonzero(sum2 == 0):
+      v2[i] = numpy.ones_like(v2[i])
+      sum2[i] = v2[i].sum()
 
-    def report(i, t): self.logger.debug('Processing entry %d', i+1 )
+    # Calculate term for shared features
+    # TODO: Possibly roll one loop back into numpy for speed
+    a = self.alpha
+    sum_ratio = (sum1.astype(float) / sum2.transpose())
+    acc_a = numpy.empty((v1.shape[0], v2.shape[0]))
+    for ip, _p in enumerate(v1):
+      for iq, _q in enumerate(v2):
+        feats = numpy.logical_and(_p, _q)
+        p = _p[feats]
+        q = _q[feats]
+        v = p * numpy.log( p / (sum_ratio[ip,iq] * a * q + (1-a) * p) )
+        acc_a[ip, iq] = numpy.sum(v)
 
-    piter = ProgressIter(n1, label = "Skew Divergence")
-    for i,r_p in enumerate(timed_report(piter,10,report)):
-      for j,(q, r_q) in enumerate(izip(n2, n3)):
-        r = r_p + r_q
-        r = r[q>0]
-        q = q[q>0]
-        results[i,j] = numpy.dot(q,numpy.log(q/r))
+    # This is just the sum of p where q is zero. 
+    acc_b  = numpy.dot(v1, (v2==0).transpose())
+
+    retval = ( acc_a + k * (acc_b + penalty) ) / sum1
 
     self.logger.debug('Returning Results')
-    return results
+    return retval
 
 class dm_outofplace(distance_metric):
   """
   Ranklist-based distance metric described in Cavnar & Trenkel 1994.
   """
   __name__ = "OutOfPlace"
-  # TODO: This could probably be further optimized for sparse data.
-  #       In particular, we should be able to again reduce the set of features
-  #       being considered. Features present in neither set will always rank tied last,
-  #       so we should be able to separately calculate their contribution to the metric
-  #       rather than naively compute it. Right now this metric is not really tractable
-  #       for big,sparse spaces.
-  
-  @staticmethod
-  def ranklist(v):
-    v = v.toarray()[0]
-    ordered_indices = reversed(numpy.argsort(v))
-    output = numpy.empty(len(v),dtype=float)
-
-    order = 1
-    indices = [ ordered_indices.next() ]
-    value = v[indices[0]] 
-
-    for index in ordered_indices: 
-      if v[index] == value:
-        indices.append(index)
-      else:
-        score_sum = sum(range(len(indices)))
-        score = order + (float(score_sum) / len(indices))
-        for i in indices:
-          output[i] = score
-
-        order = order + len(indices)
-        indices = [ index ]
-        value = v[ index ]
-
-    # finish last sequence
-    score_sum = sum(range(len(indices)))
-    score = order + (float(score_sum) / len(indices))
-    for i in indices:
-      output[i] = score
-
-    return output
-      
-  def basic(self, p, q):
-    p_rl = dm_outofplace.ranklist(p)
-    q_rl = dm_outofplace.ranklist(q)
-    result = self.fast(p_rl, q_rl) 
-    return result
-
-  def fast(self, p_rl, q_rl):
-    return numpy.sum(numpy.abs(p_rl - q_rl))
-
   def vector_distances(self, v1, v2):
-    rl_v1 = [ dm_outofplace.ranklist(v) for v in v1 ]
-    rl_v2 = [ dm_outofplace.ranklist(v) for v in v2 ]
+    # Determine shared features
+    f1 = numpy.asarray( v1.sum(0) ) [0] > 0
+    f2 = numpy.asarray( v2.sum(0) ) [0] > 0
+    no = numpy.flatnonzero(numpy.logical_or(f1, f2)) # either nonzero 
+
+    # Select only shared features from both matrices
+    v1 = v1[:,no].toarray()
+    v2 = v2[:,no].toarray()
+
+    # Compute ranklists over shared features
+    rl_v1 = [ rankdata(v) for v in v1 ]
+    rl_v2 = [ rankdata(v) for v in v2 ]
 
     results = numpy.empty((v1.shape[0],v2.shape[0]))
-    piter = ProgressIter(rl_v1, label = "Out-of-Place")
-    for i,(q_rl) in enumerate(piter):
-      for j,(r_rl) in enumerate(rl_v2):
-        results[i,j] = self.fast(q_rl, r_rl)
+    rl_v1 = ProgressIter(rl_v1, label = "Out-of-Place")
+    for i,(p_rl) in enumerate(rl_v1):
+      for j,(q_rl) in enumerate(rl_v2):
+        results[i,j] = numpy.sum(numpy.abs(p_rl-q_rl))
     return results
 
 # TODO: Tau has been dead code for some time now. Would be nice to 
