@@ -207,6 +207,7 @@ def default_configuration():
   default_config.add_section('debug')
   default_config.set('debug', 'pdb_on_classifier_exception', 'False')
   default_config.set('debug', 'pdb_on_unhandled_exception', 'False')
+  default_config.set('debug', 'pdb_on_invalid_fm', 'True')
   default_config.set('debug', 'clear_temp_files', 'True')
   default_config.set('debug', 'allow_str_classset', 'False')
   default_config.set('debug', 'allow_empty_instance', 'False')
@@ -239,7 +240,7 @@ def read_configuration(additional_path=[]):
   logger.debug("Read configuration from %s", str(paths))
   return config
 
-def update_configuration(config, requires, rescan=False, scan = []):
+def update_configuration(config, requires, rescan=False, scan = [], blank_keys=False):
   """ Receives a config object and requirements specification, then 
   scans hydrat for requirements, e.g. installed packages, tries to satisfy the 
   requirements and returns an updated configuration.
@@ -264,6 +265,8 @@ def update_configuration(config, requires, rescan=False, scan = []):
         config.set(section, key, toolpath)
       else:
         logger.info("%s --> None (not found)", key)
+        if blank_keys:
+          config.set(section, key, '')
   return config
 
 # TODO: revisit the logging infrastructure throughout hydrat
